/*
При выборе платы Digispark автоматически вкл. программный serial (если есть флаг flg_serial_debug).
Программный spi работает быстрее и занимает меньше памяти, лучше устанавливать flg_software_spi, для Tiny - это единственный вариант.
При отсутствии флага flg_serial_debug, программа быстро обрабатывает соотв. регистр и выдает данные на led (штатная работа принтера).
Все остальные режимы для настройки.
При установленном флаге flg_serial_debug, по serial port передаются данные:
flg_serial_debug = 1	Передача картинки для фокусировки обьектива, колич. байт = ARRAY_WIDTH * ARRAY_HEIGHT
flg_serial_debug = 2	Для теста точности позиционирования (идея dccharacter-а). Поиск порога как в штатном режиме (важна скорость), как только сработает порог - выдача картинки в serial port
flg_serial_debug = 3	Передача в текстовом виде на терминал данных: Max_Pix, Min_Pix, Pix_Sum, Shutter
flg_serial_debug = 4	Как 3-й режим, но по разрешению сигнала pin_TRIG (лог точно ограничен сигналом z_probe)
flg_serial_debug = 5	Данные перемещения мышки.

для работы алгоритма определения порога датчика важны:
 	#define	ConstMax		?
	#define	ConstMin		?
	#define	ConstPixMin		?
	#define	ConstPixMax		?
Для разных матриц подбирать практическим путем.

*/

#define	flg_serial_debug	1
#define	flg_software_spi

//#define	flg_ADNS_type_ADNS_5020
#define	flg_ADNS_type_ADNS_2610

#define NUM_SAMPLES_PER_MEASURE 5 //для 4-го режима

#if !defined(flg_software_spi) && !defined(__AVR_ATtiny85__)
	#include <SPI.h>
#endif

#if defined(flg_serial_debug) && defined(__AVR_ATtiny85__)
//#error "Correctly select board"
	#include <SendOnlySoftwareSerial.h>
	#define DEBUG_TX_PIN		1 //Adjust here your Tx debug pin
	SendOnlySoftwareSerial MyDbgSerial(DEBUG_TX_PIN, false); //true allows to connect to a regular RS232 without RS232 line driver
#endif

//---------------------------------------------------------------
#ifdef flg_ADNS_type_ADNS_5020

	#define ARRAY_WIDTH			15
	#define ARRAY_HEIGHT		15
	
	#define Motion				0x02
	#define Delta_X				0x03
	#define Delta_Y				0x04
	#define squal				0x05
	#define Shutter_Upper		0x06
	#define Shutter_Lower		0x07
	#define Maximum_Pixel		0x08
	#define Pixel_Sum			0x09
	#define Minimum_Pixel		0x0a
	#define Pixel_Grab			0x0b
	#define	Mask_pixel_value	0x7f
	#define	Pixel_data_valid	0x80
	#define	delay_tSWW			30
	#define	delay_tSRAD			4

	#define	ConstMax			120
	#define	ConstMin			3
	#define	ConstPixMin			50
	#define	ConstPixMax			70

#endif
#ifdef flg_ADNS_type_ADNS_2610

	#define ARRAY_WIDTH			18
	#define ARRAY_HEIGHT		18
	
	//#define Motion			0x02
	#define Delta_X				0x03
	#define Delta_Y				0x02
	#define squal				0x04
	#define Shutter_Upper		0x09
	#define Shutter_Lower		0x0a
	#define Maximum_Pixel		0x05
	#define Pixel_Sum			0x07
	#define Minimum_Pixel		0x06
	#define Pixel_Grab			0x08
	#define	Mask_pixel_value	0x3f
	#define	Pixel_data_valid	0x40
	#define	delay_tSWW			100
	#define	delay_tSRAD			100

	#define	ConstMax			59
	#define	ConstMin			3
	#define	ConstPixMin			50
	#define	ConstPixMax			70

#endif
//---------------------------------------------------------------

//---------------------------------------------------------------
// define digital io pins based on physical connection to Arduino
#ifndef __AVR_ATtiny85__
	//pin_nCS	PB2 SS D10
	#define	pin_nCS_bit		2
	#define	pin_nCS_DDR		DDRB
	#define	pin_nCS_PORT	PORTB
//-------------------------------
	//pin_led PB5 D13 аппаратный SCK
	//#define	pin_led_bit		5
	//#define	pin_led_DDR		DDRB
	//#define	pin_led_PORT	PORTB

	//pin_led PB1 D9
	#define	pin_led_bit		1
	#define	pin_led_DDR		DDRB
	#define	pin_led_PORT	PORTB
//-------------------------------

	//pin_nClock PB5 (SCK) D13
	#define	pin_nClock_bit	5
	#define	pin_nClock_DDR	DDRB
	#define	pin_nClock_PORT	PORTB

	//PB3 MOSI D11
	#define	pin_SDIO_bit		3
	#define	pin_SDIO_DDR		DDRB
	#define	pin_SDIO_PORT		PORTB
	#define	pin_SDIO_PORT_IN	PINB

#else

	//PB0(MOSI) 5 pin
	#define	pin_nCS_bit		0
	#define	pin_nCS_DDR		DDRB
	#define	pin_nCS_PORT	PORTB

	//PB1(MISO)	6pin
#if defined(flg_serial_debug)
	#define	TxPin		1
#else
	#define	pin_led_bit		1
	#define	pin_led_DDR		DDRB
	#define	pin_led_PORT	PORTB
#endif

	//PB2(SCK/ADC1) 7pin
	#define	pin_nClock_bit	2
	#define	pin_nClock_DDR	DDRB
	#define	pin_nClock_PORT	PORTB

	//PB4(usb Digispark) 3 pin
	#define	pin_SDIO_bit		4
	#define	pin_SDIO_DDR		DDRB
	#define	pin_SDIO_PORT		PORTB
	#define	pin_SDIO_PORT_IN	PINB

	//PB5(NRES) TRIGGER_PIN, 1 pin
	#define	pin_TRIG_bit		5
	#define	pin_TRIG_DDR		DDRB
	#define	pin_TRIG_PORT_IN	PINB

#endif
//---------------------------------------------------------------

#define	pin_nCS_Mode_OUTPUT		pin_nCS_DDR			|= (1<<pin_nCS_bit)
#define	pin_nCS_LOW				pin_nCS_PORT		&=~(1<<pin_nCS_bit)
#define	pin_nCS_HIGH			pin_nCS_PORT		|= (1<<pin_nCS_bit)

#if defined(flg_serial_debug) && defined(__AVR_ATtiny85__)
	#define	pin_led_Mode_OUTPUT		1
	#define	pin_led_HIGH			1
	#define	pin_led_LOW				1
	#define	pin_led_Toggle			1
#else
	#define	pin_led_Mode_OUTPUT		pin_led_DDR			|= (1<<pin_led_bit)
	#define	pin_led_HIGH			pin_led_PORT		|= (1<<pin_led_bit)
	#define	pin_led_LOW				pin_led_PORT		&=~(1<<pin_led_bit)
	#define	pin_led_Toggle			pin_led_PORT		^= (1<<pin_led_bit)
#endif

#define	pin_nClock_Mode_OUTPUT	pin_nClock_DDR		|= (1<<pin_nClock_bit)
#define	pin_nClock_LOW			pin_nClock_PORT		&=~(1<<pin_nClock_bit)
#define	pin_nClock_HIGH			pin_nClock_PORT		|= (1<<pin_nClock_bit)

#define	pin_SDIO_Mode_INPUT		pin_SDIO_DDR		&=~(1<<pin_SDIO_bit)
#define	pin_SDIO_Mode_OUTPUT	pin_SDIO_DDR		|= (1<<pin_SDIO_bit)
#define	pin_SDIO_LOW			pin_SDIO_PORT		&=~(1<<pin_SDIO_bit)
#define	pin_SDIO_HIGH			pin_SDIO_PORT		|= (1<<pin_SDIO_bit)
#define	pin_SDIO_IN				pin_SDIO_PORT_IN	&  (1<<pin_SDIO_bit)

#define	pin_TRIG_Mode_INPUT		pin_TRIG_DDR		&=~(1<<pin_TRIG_bit)
#define	pin_TRIG_IN				pin_TRIG_PORT_IN	&  (1<<pin_TRIG_bit)

#define NUM_PIXS (ARRAY_WIDTH * ARRAY_HEIGHT)
